%
%   BW_Inverse_Rate.M   This file provides a test case of the Inverse Count Rate
%                            Method for determining Blade Worth Curves
%
%   The basic test scenario is as follows:
%     1.  With a given blade worth curve, we can simulate P(t) for several distinct
%         z positions of the blade within a subcritical system.
%     2.  With the P(t) for each perturbation, we get the average count rate 
%         (i.e or average power, Pave) after stabilization 
%     3.  With the inverse Pave values, we can generate rho(z) data and do a curve 
%         fit to find the integral worth curve.  
%     4.  The test: If the computed integral worth curve is close to the initially 
%         assumed profile, then this should be a good test of the basic theory and 
%         actual implementation of the method.
%
%   This simulation uses Blade 3 as the BOI with blade worth data from Jan. 2018...
%
%   Let's give this a try...  
%   
%   File prepared J. R. White, UMass-Lowell (last update:  March 2018)
%

      clear all,  close all,  nfig = 0;  
%%
%   define UMLRR-specific kinetics data and other parameters for the simulations
      [Be,B,Gt,lam,td,S,kappa,nu] = kinetics_data();
      tol  = 1e-5;  options = odeset('RelTol',tol); % error tolerance for ODE solver
      ftime = 600;            % final simulation time (seconds)
      dt = 1.0;               % sampling time (sec)
      t = (0:dt:ftime)';      % time vector with sampling time set by dt
      nlevel = 0.10;          % random noise level (random uncertainty in P(t) data)
%
      rho0 = -5*Be;  Q = S;   % use 5 dollars subcritical & constant source inserted
      Po = -kappa*Q/nu/rho0;  % initial power level
      Co = B*Po./(lam*Gt);    % initial precursor conc
      boi = 3;  yr = 'Jan2018';    % blade of interest (Blade 3) & year of worth data 
      [dum,H] = bworth(0,boi,yr);  % simply returns the max height for the BOI
      rate = 3.7/60;          % rate of blade movement for BOI (inches/sec)
      zlocs = [0 4 8 10 12 13 14 16 18 22 H];  % blade locations (inches withdrawn)
%
%%
%   do simulation for each dz (must be piecewise linear)
      Nz = length(zlocs);   rho = zeros(Nz,1);  CR = rho;
      CR(1) = Po;               % initial count rate
      nfig = nfig+1;   figure(nfig)  % open figure prior to loop
      for i = 1:Nz-1
        xo = [Po Co]';          % initial condition for state vector
        ftx = @(t,x) pkeqns_rampb_nofdbk(t,x,rho0,Q,Be,Gt,B,lam,kappa,nu, ...
                                         rate,zlocs(i:i+1),boi,yr);
        [t,x] = ode15s(ftx,t,xo,options);
        P = x(:,1);             % do not normalize P(t)
%   add some noise to the P(t) signal
        rn = rand(size(t));     % vector of uniform random numbers (0 to 1)
        a = 1-nlevel;   b = 1+nlevel;  % nlevel is the 'noise level'  (see above)
        srn = (b-a)*rn + a;     % uniformly distributed between a and b
        srn(1) = 1.0;           % reset initial value to 1.0
        Pn = srn.*P;            % P(t) with random noise
%   compute new count rate after stablization & set initial conditions for next step
        CR(i+1) = mean(Pn(end-300:end));  % new steady state power level (last 300 s)  
        drho = bworth(zlocs(i+1),boi,yr)-bworth(zlocs(i),boi,yr);  % actual delrho
        drho = drho/100;                  % actual delrho in Dk/k
        rho0 = rho0 + drho;               % new rho0 for next step 
        Po = -kappa*Q/nu/rho0;            % initial power level
        Co = B*Po./(lam*Gt);              % initial precursor conc
%   plot P(t) and z(t) profiles
        if i == 1; tt = t;  else  tt = tt+ftime;  end
        z = zlocs(i)+rate*t;  z(z > zlocs(i+1)) = zlocs(i+1); 
        subplot(2,1,1),plot(tt/60,Pn/CR(1),'r-','LineWidth',2),grid on, hold on
        title('Inverse\_Rate: Simulated Count Rate + Blade Position vs Time')
        ylabel('Normalized Count Rate')
        subplot(2,1,2),plot(tt/60,z,'b-','LineWidth',2),grid on, hold on
        xlabel('Time (minutes)'), ylabel('Blade Position (inches out)')
      end
%%
%   at this point, one needs the following vectors to be available for the curve fit:
%      zlocs -- vector containing the actual blade locations
%      CR  --  steady state detector count rate corresponding to the blade locations
%
%  now let's compute the integral worth at each z from the inverse count rate data
      CR = CR/CR(1);   icin = 1/CR(1);   icout = 1/CR(end);   ic = 1./CR;
      rhown = (ic - icin)/(icout - icin);   % normalized worth vector
%%
%   use Matlab's built-in fitting routines to do the curve fit for the integral
%   worth to the Combined Polynomial and Sinusoid Model (PSM for short)
%    math model:  rho(z)/rhotot = c1*z^4 + c2*z^3 + c3*z^2 + c4*z + c5 + c6*sin(2*pi*z/H)
      expr = {'x.^4','x.^3','x.^2','x','1','sin(2*pi*x)'}; % x is normalized height (z/H)
      PSM = fittype(expr);                          % setup for Matlab's 'fit' command
      options = fitoptions('Method','LinearLeastSquares','Normalize','off');
      x = zlocs';  y = rhown;                       % data for curve fit
      [cfun,good] = fit(x/H,y,PSM,options);         % do actual curve fit 
      c = coeffvalues(cfun);                        % normalized coeffs of math model 
      r2 = good.rsquare;                            % coeff of determination (i.e. r^2)
      c(1) = c(1)/H^4;  c(2) = c(2)/H^3;            % coeffs for unnormalized model
      c(3) = c(3)/H^2;  c(4) = c(4)/H;  
%
      NN = 100;    xx = linspace(0,H,NN);                    % z values for plotting
      yy = polyval(c(1:5),xx) + c(6)*sin(2*pi*xx/H);         % measured normalized rho(z)
      rhotot = bworth(H,boi,yr)-bworth(0,boi,yr);            % actual total worth (%Dk/k)
      rhoactn = (bworth(xx,boi,yr)-bworth(0,boi,yr))/rhotot; % actual normalized rho(z) 
%%
%   plot shape of integral worth curves
      nfig = nfig + 1; figure(nfig)
      plot(x,y,'ro',xx,yy,'r-',xx,rhoactn,'b.','Linewidth',2),grid
      title(['Inverse\_Rate: Blade #',num2str(boi), ...
             ' Normalized Integral Worth with r^2 = ',num2str(r2,'%5.2f')])
      xlabel('Blade Withdrawal Height (in)'),ylabel('Normalized Reactivity Worth')
      legend('Simulated Data Points','Curve Fit to Simulated Data', ...
             ['Actual Worth Curve (\rho_{tot} = ',num2str(rhotot,'%4.2f'), ...
              ' %\Deltak/k)'],'location','southeast')
      axis([0 26 0 1.01])
%
%   end of program
